/*
 * This include files defines the interface of the native functions in 
 * corba_wrappers.c. It is included by GHC when it compiles the GreenCard
 * generated code via the C compiler. That's why there are only C declarations
 * here.
 *
 */

#ifndef __CORBA_WRAPPERS_H__
#define __CORBA_WRAPPERS_H__

/*
 * Simple C types for the CORBA types.
 */

typedef char * CStringArray;
typedef void * CORBA_ORB_ptr;
typedef void * CORBA_Object_ptr;
typedef void * CORBA_StructMemberSeq_ptr;
typedef void * CORBA_TypeCode_ptr;
typedef void * CORBA_UnionMemberSeq_ptr;
typedef void * CORBA_EnumMemberSeq_ptr;
typedef void * CORBA_Request_ptr;
typedef void * CORBA_Any_ptr;
typedef void * DynamicAny_AnySeq_ptr;
typedef void * DynamicAny_NameValuePairSeq_ptr;
typedef void * CORBA_NamedValue_ptr;
typedef void * CORBA_NVList_ptr;
typedef void * CORBA_Exception_ptr;
typedef void * CORBA_UnknownUserException_ptr;
typedef void * CORBA_ServerRequest_ptr;
typedef int CORBA_TCKind;
typedef unsigned long CORBA_ULong;
typedef int bool;

/******************************/
/*       CStringArray         */
/******************************/

CStringArray
c_string_array_create(int size);

void 
c_string_array_set(CStringArray arr, int index, char * value);

char * 
c_string_array_get(CStringArray arr, 
				   int index);

void
c_string_array_delete(CStringArray arr);

/******************************/
/*          ORB               */
/******************************/

CORBA_ORB_ptr
corba_orb_init(CStringArray argv, int argc,
			   CStringArray * out_argv, int * out_argc);

CORBA_Object_ptr
corba_orb_string_to_object(CORBA_ORB_ptr orb, 
						   char * str);

CORBA_Object_ptr
corba_orb_bind(CORBA_ORB_ptr orb, 
			   char * repoid, 
			   char * address);

/******************************/
/*        OBJECT              */
/******************************/

char *
corba_orb_object_to_string(CORBA_ORB_ptr orb,
						   CORBA_Object_ptr obj);

CORBA_Object_ptr
corba_orb_resolve_initial_references(CORBA_ORB_ptr orb,
									 char * name);

CORBA_Request_ptr
corba_object_create_request(CORBA_Object_ptr obj,
							char * operation);

int
corba_object_is_a(CORBA_Object_ptr obj,
				  char * repoid);

CORBA_Object_ptr
corba_object_nil();

int
corba_object_hash(CORBA_Object_ptr obj,
				  CORBA_ULong max);

/******************************/
/*         TYPECODE           */
/******************************/


CORBA_StructMemberSeq_ptr 
corba_struct_member_seq_create(int size);

void
corba_struct_member_seq_fill(CORBA_StructMemberSeq_ptr seq,
							 int index,
							 char * name,
							 CORBA_TypeCode_ptr type);

void
corba_struct_member_seq_delete(CORBA_StructMemberSeq_ptr seq);

CORBA_UnionMemberSeq_ptr
corba_union_member_seq_create(int size);

void
corba_union_member_seq_fill(CORBA_UnionMemberSeq_ptr seq,
							int index,
							char * name,
							CORBA_Any_ptr label,
							CORBA_TypeCode_ptr type);

void
corba_union_member_seq_delete(CORBA_UnionMemberSeq_ptr seq);

CORBA_EnumMemberSeq_ptr
corba_enum_member_seq_create(int size);

void
corba_enum_member_seq_fill(CORBA_EnumMemberSeq_ptr seq,
						   int index,
						   char * name);

void
corba_enum_member_seq_delete(CORBA_EnumMemberSeq_ptr seq);

CORBA_TypeCode_ptr
corba_tc_create_basic(CORBA_TCKind kind);

CORBA_TypeCode_ptr
corba_tc_create_string(int bound);

CORBA_TypeCode_ptr
corba_tc_create_objref(char * repoid, 
					   char * name);

CORBA_TypeCode_ptr
corba_tc_create_struct(char * repoid, 
					   char * name,
					   CORBA_StructMemberSeq_ptr members);

CORBA_TypeCode_ptr
corba_tc_create_union(char * repoid, 
					  char * name,
					  CORBA_TypeCode_ptr label_type,
					  CORBA_UnionMemberSeq_ptr members);

CORBA_TypeCode_ptr
corba_tc_create_sequence(int bound,
						 CORBA_TypeCode_ptr content_type);

CORBA_TypeCode_ptr
corba_tc_create_enum(char * repoid, 
					 char * name,
					 CORBA_EnumMemberSeq_ptr members);

CORBA_TypeCode_ptr
corba_tc_create_alias(char * repoid, 
					  char * name,
					  CORBA_TypeCode_ptr content_type);

CORBA_TypeCode_ptr
corba_tc_create_array(CORBA_TypeCode_ptr content_type,
					  int length);

CORBA_TypeCode_ptr
corba_tc_create_exception(char * repoid, 
						  char * name,
						  CORBA_StructMemberSeq_ptr members);

CORBA_TypeCode_ptr
corba_tc_create_recursive(char * repoid);

void
corba_tc_release(CORBA_TypeCode_ptr tc);

int
corba_tc_get_kind(CORBA_TypeCode_ptr tc);

char *
corba_tc_get_id(CORBA_TypeCode_ptr tc);

char *
corba_tc_get_name(CORBA_TypeCode_ptr tc);

int
corba_tc_get_member_count(CORBA_TypeCode_ptr tc);

char *
corba_tc_get_member_name(CORBA_TypeCode_ptr tc,
						 int index);

CORBA_TypeCode_ptr
corba_tc_get_member_type(CORBA_TypeCode_ptr tc,
						 int index);

CORBA_Any_ptr
corba_tc_get_member_label(CORBA_TypeCode_ptr tc,
						  int index);

CORBA_TypeCode_ptr 
corba_tc_get_discriminator_type(CORBA_TypeCode_ptr tc);

int
corba_tc_get_default_index(CORBA_TypeCode_ptr tc);

int
corba_tc_get_length(CORBA_TypeCode_ptr tc);

CORBA_TypeCode_ptr
corba_tc_get_content_type(CORBA_TypeCode_ptr tc);

/******************************/
/*            ANY             */
/******************************/

CORBA_Any_ptr 
corba_any_create_null();

CORBA_Any_ptr 
corba_any_create_short(int value);

CORBA_Any_ptr 
corba_any_create_ushort(int value);

CORBA_Any_ptr 
corba_any_create_long(int value);

CORBA_Any_ptr 
corba_any_create_ulong(int value);

CORBA_Any_ptr 
corba_any_create_float(double value);

CORBA_Any_ptr 
corba_any_create_double(double value);

CORBA_Any_ptr 
corba_any_create_bool(int value);

CORBA_Any_ptr 
corba_any_create_char(int value);

CORBA_Any_ptr 
corba_any_create_octet(int value);

CORBA_Any_ptr 
corba_any_create_string(char * value);

CORBA_Any_ptr 
corba_any_create_any(CORBA_Any_ptr value);

CORBA_Any_ptr 
corba_any_create_tc(CORBA_TypeCode_ptr value);

CORBA_Any_ptr 
corba_any_create_objref(CORBA_Object_ptr value);

void
corba_any_delete(CORBA_Any_ptr any);

CORBA_TypeCode_ptr
corba_any_get_type(CORBA_Any_ptr any);

int
corba_any_extract_short(CORBA_Any_ptr any);

int
corba_any_extract_long(CORBA_Any_ptr any);

int
corba_any_extract_ushort(CORBA_Any_ptr any);

int
corba_any_extract_ulong(CORBA_Any_ptr any);

double
corba_any_extract_float(CORBA_Any_ptr any);

double
corba_any_extract_double(CORBA_Any_ptr any);

bool
corba_any_extract_boolean(CORBA_Any_ptr any);

char
corba_any_extract_char(CORBA_Any_ptr any);

int
corba_any_extract_octet(CORBA_Any_ptr any);

char *
corba_any_extract_string(CORBA_Any_ptr any);

const CORBA_Any_ptr
corba_any_extract_any(CORBA_Any_ptr any);

CORBA_TypeCode_ptr
corba_any_extract_tc(CORBA_Any_ptr any);

CORBA_Object_ptr
corba_any_extract_objref(CORBA_Any_ptr any);

/******************************/
/*        DYNANY              */
/******************************/

DynamicAny_NameValuePairSeq_ptr
corba_nv_pair_seq_create(int size);

void
corba_nv_pair_seq_fill(DynamicAny_NameValuePairSeq_ptr seq,
					   int index,
					   char * name,
					   CORBA_Any_ptr value);

int
corba_nv_pair_seq_length(DynamicAny_NameValuePairSeq_ptr seq);

void
corba_nv_pair_seq_get(DynamicAny_NameValuePairSeq_ptr seq,
					  int index,
					  char * * out_name,
					  CORBA_Any_ptr* out_value);

void
corba_nv_pair_seq_delete(DynamicAny_NameValuePairSeq_ptr seq);

CORBA_Any_ptr
corba_dyn_struct_create(CORBA_TypeCode_ptr tc,
						DynamicAny_NameValuePairSeq_ptr members);

DynamicAny_NameValuePairSeq_ptr
corba_dyn_struct_members(CORBA_Any_ptr any);

DynamicAny_AnySeq_ptr
corba_any_seq_create(int size);

void
corba_any_seq_fill(DynamicAny_AnySeq_ptr seq,
				   int index,
				   CORBA_Any_ptr value);

int
corba_any_seq_length(DynamicAny_AnySeq_ptr seq);

CORBA_Any_ptr
corba_any_seq_get(DynamicAny_AnySeq_ptr seq,
				  int index);

void
corba_any_seq_delete(DynamicAny_AnySeq_ptr seq);

CORBA_Any_ptr
corba_dyn_sequence_create(CORBA_TypeCode_ptr tc,
						  DynamicAny_AnySeq_ptr elements);

DynamicAny_AnySeq_ptr
corba_dyn_sequence_elements(CORBA_Any_ptr any);

CORBA_Any_ptr
corba_dyn_union_create(CORBA_TypeCode_ptr tc,
					   CORBA_Any_ptr label,
					   CORBA_Any_ptr value);

void
corba_dyn_union_get(CORBA_Any_ptr any,
					CORBA_Any_ptr* out_label,
					CORBA_Any_ptr* out_member);

CORBA_Any_ptr
corba_dyn_enum_create(CORBA_TypeCode_ptr tc,
					  char * value);

char *
corba_dyn_enum_get(CORBA_Any_ptr any);

CORBA_Any_ptr
corba_dyn_array_create(CORBA_TypeCode_ptr tc,
					   DynamicAny_AnySeq_ptr elements);

DynamicAny_AnySeq_ptr
corba_dyn_array_elements(CORBA_Any_ptr any);

/******************************/
/*       NAMEDVALUE           */
/******************************/

char *
corba_nv_get_name(CORBA_NamedValue_ptr nv);

CORBA_Any_ptr
corba_nv_get_value(CORBA_NamedValue_ptr nv);

int
corba_nv_get_flags(CORBA_NamedValue_ptr nv);

void
corba_nv_set_type(CORBA_NamedValue_ptr nv,
				  CORBA_TypeCode_ptr tc);

void
corba_nv_set_value(CORBA_NamedValue_ptr nv,
				   CORBA_Any_ptr value);

/******************************/
/*           NVLIST           */
/******************************/

CORBA_NVList_ptr
corba_nvlist_create();

CORBA_NamedValue_ptr
corba_nvlist_add(CORBA_NVList_ptr nvlist,
				 int flags);

int
corba_nvlist_count(CORBA_NVList_ptr nvlist);

CORBA_NamedValue_ptr
corba_nvlist_get(CORBA_NVList_ptr nvlist,
				 int index);

/******************************/
/*       REQUEST              */
/******************************/

CORBA_NVList_ptr
corba_request_arguments(CORBA_Request_ptr req);

void 
corba_request_add_in_arg(CORBA_Request_ptr req,
						 char * name,
						 CORBA_Any_ptr value);

void 
corba_request_add_out_arg(CORBA_Request_ptr req,
						  char * name,
						  CORBA_Any_ptr value);

void 
corba_request_add_inout_arg(CORBA_Request_ptr req,
							char * name,
							CORBA_Any_ptr value);

void 
corba_request_add_out_arg_template(CORBA_Request_ptr req,
								   char * name,
								   CORBA_TypeCode_ptr tc);

void 
corba_request_invoke(CORBA_Request_ptr req);

void 
corba_request_set_result_tc(CORBA_Request_ptr req,
							CORBA_TypeCode_ptr tc);
  
CORBA_Any_ptr
corba_request_result(CORBA_Request_ptr req);

CORBA_Exception_ptr
corba_request_exception(CORBA_Request_ptr req);

void
corba_request_add_exception_def(CORBA_Request_ptr req,
								CORBA_TypeCode_ptr tc);

void
corba_request_delete(CORBA_Request_ptr req);

/******************************/
/*    EXCEPTION               */
/******************************/

int
corba_exception_is_system(CORBA_Exception_ptr ex);

int
corba_exception_get_minor(CORBA_Exception_ptr ex);

int
corba_exception_get_completed(CORBA_Exception_ptr ex);

char *
corba_exception_get_repoid(CORBA_Exception_ptr ex);

CORBA_UnknownUserException_ptr
corba_unknown_ex_narrow(CORBA_Exception_ptr ex);

char *
corba_unknown_ex_get_repoid(CORBA_UnknownUserException_ptr ex);

CORBA_Any_ptr
corba_unknown_ex_get_value(CORBA_UnknownUserException_ptr ex,
						   CORBA_TypeCode_ptr tc);

/******************************/
/*      SERVER REQUEST        */
/******************************/

char *
corba_server_request_operation(CORBA_ServerRequest_ptr req);

void
corba_server_request_set_params(CORBA_ServerRequest_ptr req,
								CORBA_NVList_ptr args);

void
corba_server_request_set_result(CORBA_ServerRequest_ptr req,
								CORBA_Any_ptr result);

void
corba_server_request_set_exception(CORBA_ServerRequest_ptr req,
								   CORBA_Any_ptr result);

CORBA_NVList_ptr
corba_server_request_get_args(CORBA_ServerRequest_ptr req);

CORBA_Object_ptr
corba_servant_create(char * repoid);

void corba_server_run(void * callback_ptr, void * a_ps_ptr);

void corba_server_exit();

#endif

